/*******************************************************************************




     Alarme de abertura de porta e janela

     C/ PIC 12F675

     Sensor com Reed Switch e im

     Autor: Cludio Lrios

     Data: 28/09/2016


     Uso didtico

     Este arquivo  parte integrante do blog Lrios.Tecnologia.WS

 *****************************************************************************/

#include <12F675.h>
#device adc=8
#use delay(clock=4000000)
#fuses NOWDT,INTRC_IO, NOCPD, NOPROTECT, NOMCLR, NOPUT, NOBROWNOUT//configurao

//bits
#bit WPU5          = 0X95.5 //pull up pino 2
#bit GPIF          = 0x0b.0 //flag da int. mudana
#bit REEDSWITCH    = 0x5.5   //reed switch na porta
#bit ON            = 0x5.3   //liga/desliga
#bit IOC5          = 0X96.5 //interrupo por mudana no pino 2
#bit IOC3          = 0x96.3 //interrupo por mudana no pino 4
#bit tmr0if        = 0x0b.2 //flag de overflow do timer 0

//bytes
#byte OPTION_REG = 0x81
#byte TRISIO = 0x85
#byte GPIO   = 0x05
#byte INTCON  = 0x0B
#byte TMR0    = 0x01

//defines
#define _A   284    //nota musical 1
#define _B   253    //nota musical 2

//variveis globais
unsigned int1 f_x=0;
unsigned int8 t_init,t_end;

/******************************************************************************
               SUBROTINAS
*******************************************************************************/


/******************************************************************************
    ROTINA PARA LEITURA DA TENSO DOS TRIMPOTS DE TEMPO INICIAL E FINAL
*******************************************************************************/
void le_pot(){
   set_adc_channel(2); //ajusta para o pino 5 (AN2)
   delay_us(200); //delay para normalizar a tenso no pino a ser lido
    t_init=READ_ADC();// l valor da tenso de entrada do pino 5
   set_adc_channel(3); //ajusta para o pino 3 (AN3)
   delay_us(200); //delay para normalizar a tenso no pino a ser lido
    t_end=READ_ADC();// l valor da tenso de entrada do pino 5
}
/******************************************************************************
               ROTINA DE DELAY INICIAL
*******************************************************************************/

void delay_init(){
 unsigned int8 a;
    for(a=0;a<t_init;a++){
     delay_ms(500);// 0,5 segundos
     if(!ON){GPIO=0;break;}//sai da rotina se on/off for desligado
    }
}

/******************************************************************************
               ROTINA PRINCIPAL (MAIN)
*******************************************************************************/

void main() {
   #define N_REP 3
   unsigned int8 a,b;
  
   setup_adc_ports(sAN2|sAN3 | VSS_VDD);  //seleciona AN2 e AN3 com 5v como referncia
   setup_adc(ADC_CLOCK_DIV_8 );// ajusta o clock do adc
   set_adc_channel(2); //ajusta para o pino 5 (AN2)
   setup_counters(RTCC_INTERNAL,RTCC_DIV_1);
   setup_timer_1(T1_DISABLED);
   setup_comparator(NC_NC_NC_NC);
   setup_vref(FALSE);
   TRISIO= 0b111100;//somente pinos 5 e 6 como sadas
   WPU5=1;//habilita pullup no pino 2   
   OPTION_REG=0;//liga pullup geral
   INTCON= 0B00001000;//liga interrupo por mudana de nvel nos pinos
   IOC5=1; //liga interrupo individual por mudana no pino 2
   IOC3=1; //liga interrupo individual por mudana no pino 4
/******************************************************************************
              LAO DE REPETIO
*******************************************************************************/
   
   for(;;){

    
     GPIO=0;
     GPIF=0;
     sleep();//entra em modo de baixo consumo
  //..................................ZZZZZZZZZZZZZZZZZZZZZZZZZZZ   
     
     delay_us(1);//nop ps sleep
     le_pot();//atualiza valores para delays aps pic ser 'acordado'
     
     if(!f_x){//usa um led para indicar que esta no periodo de delay inicial
       GPIO=1;//liga led indicador de tempo inicial
       delay_init();
     }
     GPIO=0;//desliga led indicador de tempo inicial
    
     
     if(ON && f_x){// se ligado e se passou o tempo inicial ir disparar
      while(t_end||ON){//enquanto t_end>0 e on/off ligada
       for(b=0;b<N_REP;b++){
         for(a=0;a<250;a++){  //gera o sinal de alarme: nota A
          GPIO= 0b000010;//aciona o piezo eltrico em um sentido
          delay_us(_A);
          GPIO= 0b000001;//aciona o piezo eltrico no outro sentido
          delay_us(_A);
         }
          for(a=0;a<200;a++){//gera o sinal de alarme: nota B
          GPIO= 0b000010;
          delay_us(_B);
          GPIO= 0b000001;
          delay_us(_B);
         }
       }
       if(!ON){f_x=0;if(t_end)t_end--;}//decrementa se for p/ contar tempo final
     } //while(t_end)
    }//if(ON)
     
     if(!ON) f_x=0; else f_x=1;
     delay_ms(50);  
   }//for(;;);
}//main()
//-----------------------------------------fim----------------------------------
